@@ -36,7 +36,7 @@ except ImportError:
     pymysql = None
 import six
 import sqlalchemy
-import sqlalchemy.engine.url as sqlalchemy_url
+from sqlalchemy.engine import url as sqlalchemy_url
 import sqlalchemy.exc
 from sqlalchemy import types
 import sqlalchemy_utils
@@ -60,13 +60,9 @@ LOG = log.getLogger(__name__)
 
 
 def retry_on_deadlock(f):
-    # FIXME(jd) The default values in oslo.db are useless, we need to fix that.
-    # Once it's done, let's remove that wrapper of wrapper.
     return oslo_db.api.wrap_db_retry(retry_on_deadlock=True,
-                                     retry_on_request=True,
-                                     max_retries=10,
+                                     max_retries=20,
                                      retry_interval=0.1,
-                                     inc_retry_interval=True,
                                      max_retry_interval=2)(f)
 
 
@@ -112,99 +108,131 @@ class ResourceClassMapper(object):
     def __init__(self):
         self._cache = {'generic': {'resource': base.Resource,
                                    'history': base.ResourceHistory}}
-        self._lock = threading.RLock()
 
     @staticmethod
     def _build_class_mappers(resource_type, baseclass=None):
         tablename = resource_type.tablename
+        tables_args = {"extend_existing": True}
+        tables_args.update(base.COMMON_TABLES_ARGS)
         # TODO(sileht): Add columns
         if not baseclass:
             baseclass = resource_type.to_baseclass()
         resource_ext = type(
             str("%s_resource" % tablename),
             (baseclass, base.ResourceExtMixin, base.Resource),
-            {"__tablename__": tablename})
+            {"__tablename__": tablename, "__table_args__": tables_args})
         resource_history_ext = type(
             str("%s_history" % tablename),
             (baseclass, base.ResourceHistoryExtMixin, base.ResourceHistory),
-            {"__tablename__": ("%s_history" % tablename)})
+            {"__tablename__": ("%s_history" % tablename),
+             "__table_args__": tables_args})
         return {'resource': resource_ext,
                 'history': resource_history_ext}
 
     def get_classes(self, resource_type):
-        # NOTE(sileht): Most of the times we can bypass the lock so do it
+        # NOTE(sileht): We don't care about concurrency here because we allow
+        # sqlalchemy to override its global object with extend_existing=True
+        # this is safe because classname and tablename are uuid.
         try:
             return self._cache[resource_type.tablename]
         except KeyError:
-            pass
-        # TODO(sileht): if the table doesn't exis
-        with self._lock:
-            try:
-                return self._cache[resource_type.tablename]
-            except KeyError:
-                mapper = self._build_class_mappers(resource_type)
-                self._cache[resource_type.tablename] = mapper
-                return mapper
+            mapper = self._build_class_mappers(resource_type)
+            self._cache[resource_type.tablename] = mapper
+            return mapper
 
     @retry_on_deadlock
     def map_and_create_tables(self, resource_type, facade):
-        with self._lock:
-            # NOTE(sileht): map this resource_type to have
-            # Base.metadata filled with sa.Table objects
-            mappers = self.get_classes(resource_type)
-            tables = [Base.metadata.tables[klass.__tablename__]
-                      for klass in mappers.values()]
+        if resource_type.state != "creating":
+            raise RuntimeError("map_and_create_tables must be called in state "
+                               "creating")
+
+        mappers = self.get_classes(resource_type)
+        tables = [Base.metadata.tables[klass.__tablename__]
+                  for klass in mappers.values()]
+
+        try:
+            with facade.writer_connection() as connection:
+                Base.metadata.create_all(connection, tables=tables)
+        except exception.DBError as e:
+            if self._is_current_transaction_aborted(e):
+                raise exception.RetryRequest(e)
+            raise
+
+        # NOTE(sileht): no need to protect the _cache with a lock
+        # get_classes cannot be called in state creating
+        self._cache[resource_type.tablename] = mappers
+
+    @staticmethod
+    def _is_current_transaction_aborted(exception):
+        # HACK(jd) Sometimes, PostgreSQL raises an error such as "current
+        # transaction is aborted, commands ignored until end of transaction
+        # block" on its own catalog, so we need to retry, but this is not
+        # caught by oslo.db as a deadlock. This is likely because when we use
+        # Base.metadata.create_all(), sqlalchemy itself gets an error it does
+        # not catch or something. So this is why this function exists. To
+        # paperover I guess.
+        inn_e = exception.inner_exception
+        return (psycopg2
+                and isinstance(inn_e, sqlalchemy.exc.InternalError)
+                and isinstance(inn_e.orig, psycopg2.InternalError)
+                # current transaction is aborted
+                and inn_e.orig.pgcode == '25P02')
+
+    @retry_on_deadlock
+    def unmap_and_delete_tables(self, resource_type, connection):
+        if resource_type.state != "deleting":
+            raise RuntimeError("unmap_and_delete_tables must be called in "
+                               "state deleting")
+
+        mappers = self.get_classes(resource_type)
+        del self._cache[resource_type.tablename]
+
+        tables = [Base.metadata.tables[klass.__tablename__]
+                  for klass in mappers.values()]
+
+        if connection is not None:
+            # NOTE(sileht): Base.metadata.drop_all doesn't
+            # issue CASCADE stuffs correctly at least on postgresql
+            # We drop foreign keys manually to not lock the destination
+            # table for too long during drop table.
+            # It's safe to not use a transaction since
+            # the resource_type table is already cleaned and commited
+            # so this code cannot be triggerred anymore for this
+            # resource_type
             try:
-                with facade.writer_connection() as connection:
-                    Base.metadata.create_all(connection, tables=tables)
+                for table in tables:
+                    for fk in table.foreign_key_constraints:
+                        try:
+                            self._safe_execute(
+                                connection,
+                                sqlalchemy.schema.DropConstraint(fk))
+                        except exception.DBNonExistentConstraint:
+                            pass
+                for table in tables:
+                    try:
+                        self._safe_execute(connection,
+                                           sqlalchemy.schema.DropTable(table))
+                    except exception.DBNonExistentTable:
+                        pass
             except exception.DBError as e:
-                # HACK(jd) Sometimes, PostgreSQL raises an error such as
-                # "current transaction is aborted, commands ignored until end
-                # of transaction block" on its own catalog, so we need to
-                # retry, but this is not caught by oslo.db as a deadlock. This
-                # is likely because when we use Base.metadata.create_all(),
-                # sqlalchemy itself gets an error it does not catch or
-                # something. So this is paperover I guess.
-                inn_e = e.inner_exception
-                if (psycopg2
-                   and isinstance(inn_e, sqlalchemy.exc.InternalError)
-                   and isinstance(inn_e.orig, psycopg2.InternalError)
-                    # current transaction is aborted
-                   and inn_e.orig.pgcode == '25P02'):
+                if self._is_current_transaction_aborted(e):
                     raise exception.RetryRequest(e)
                 raise
 
-    def unmap_and_delete_tables(self, resource_type, connection):
-        with self._lock:
-            # NOTE(sileht): map this resource_type to have
-            # Base.metadata filled with sa.Table objects
-            mappers = self.get_classes(resource_type)
-            tables = [Base.metadata.tables[klass.__tablename__]
-                      for klass in mappers.values()]
+            # NOTE(sileht): If something goes wrong here, we are currently
+            # fucked, that why we expose the state to the superuser.
+            # TODO(sileht): The idea is to make the delete resource_type more
+            # like a cleanup method, I mean we should don't fail if the
+            # constraint have already been dropped or the table have already
+            # been deleted. So, when the superuser have fixed it's backend
+            # issue, it can rerun 'DELETE ../resource_type/foobar' even the
+            # state is already error and if we are sure all underlying
+            # resources have been cleaned we really deleted the resource_type.
 
-            if connection is not None:
-                # NOTE(sileht): Base.metadata.drop_all doesn't
-                # issue CASCADE stuffs correctly at least on postgresql
-                # We drop foreign keys manually to not lock the destination
-                # table for too long during drop table.
-                # It's safe to not use a transaction since
-                # the resource_type table is already cleaned and commited
-                # so this code cannot be triggerred anymore for this
-                # resource_type
-                for table in tables:
-                    for fk in table.foreign_key_constraints:
-                        self._safe_execute(
-                            connection,
-                            sqlalchemy.schema.DropConstraint(fk))
-                for table in tables:
-                    self._safe_execute(connection,
-                                       sqlalchemy.schema.DropTable(table))
-
-            # TODO(sileht): Remove this resource on other workers
-            # by using expiration on cache ?
-            for table in tables:
-                Base.metadata.remove(table)
-            del self._cache[resource_type.tablename]
+        # TODO(sileht): Remove this resource on other workers
+        # by using expiration on cache ?
+        for table in tables:
+            Base.metadata.remove(table)
 
     @retry_on_deadlock
     def _safe_execute(self, connection, works):
@@ -221,14 +249,27 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     @classmethod
     def _create_new_database(cls, url):
         """Used by testing to create a new database."""
-        purl = sqlalchemy_url.make_url(url)
+        purl = sqlalchemy_url.make_url(
+            cls.dress_url(
+                url))
         purl.database = purl.database + str(uuid.uuid4()).replace('-', '')
         new_url = str(purl)
         sqlalchemy_utils.create_database(new_url)
         return new_url
 
+    @staticmethod
+    def dress_url(url):
+        # If no explicit driver has been set, we default to pymysql
+        if url.startswith("mysql://"):
+            url = sqlalchemy_url.make_url(url)
+            url.drivername = "mysql+pymysql"
+            return str(url)
+        return url
+
     def __init__(self, conf):
-        conf.set_override("connection", conf.indexer.url, "database")
+        conf.set_override("connection",
+                          self.dress_url(conf.indexer.url),
+                          "database")
         self.conf = conf
         self.facade = PerInstanceFacade(conf)
 
@@ -271,12 +312,24 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         for rt in base.get_legacy_resource_types():
             if not (rt.name == "generic" or create_legacy_resource_types):
                 continue
+
             try:
                 with self.facade.writer() as session:
                     session.add(rt)
             except exception.DBDuplicateEntry:
-                pass
-            self._RESOURCE_TYPE_MANAGER.map_and_create_tables(rt, self.facade)
+                continue
+
+            if rt.name != "generic":
+                try:
+                    self._RESOURCE_TYPE_MANAGER.map_and_create_tables(
+                        rt, self.facade)
+                except Exception:
+                    self._set_resource_type_state(rt.name, "creation_error")
+                    LOG.exception('Fail to create tables for '
+                                  'resource_type "%s"', rt.name)
+                    continue
+
+            self._set_resource_type_state(rt.name, "active")
 
     # NOTE(jd) We can have deadlock errors either here or later in
     # map_and_create_tables(). We can't decorate create_resource_type()
@@ -296,12 +349,13 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         # NOTE(sileht): mysql have a stupid and small length limitation on the
         # foreign key and index name, so we can't use the resource type name as
         # tablename, the limit is 64. The longest name we have is
-        # fk_<tablename>_history_revision_resource_history_revision,
-        # so 64 - 46 = 18
-        tablename = "rt_%s" % uuid.uuid4().hex[:15]
+        # fk_<tablename>_h_revision_rh_revision,
+        # so 64 - 26 = 38 and 3 chars for rt_, 35 chars, uuid is 32, it's cool.
+        tablename = "rt_%s" % uuid.uuid4().hex
         resource_type = ResourceType(name=resource_type.name,
                                      tablename=tablename,
-                                     attributes=resource_type.attributes)
+                                     attributes=resource_type.attributes,
+                                     state="creating")
 
         # NOTE(sileht): ensure the driver is able to store the request
         # resource_type
@@ -309,9 +363,17 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
         self._add_resource_type(resource_type)
 
-        self._RESOURCE_TYPE_MANAGER.map_and_create_tables(resource_type,
-                                                          self.facade)
+        try:
+            self._RESOURCE_TYPE_MANAGER.map_and_create_tables(resource_type,
+                                                              self.facade)
+        except Exception:
+            # NOTE(sileht): We fail the DDL, we have no way to automatically
+            # recover, just set a particular state
+            self._set_resource_type_state(resource_type.name, "creation_error")
+            raise
 
+        self._set_resource_type_state(resource_type.name, "active")
+        resource_type.state = "active"
         return resource_type
 
     def get_resource_type(self, name):
@@ -324,6 +386,17 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             raise indexer.NoSuchResourceType(name)
         return resource_type
 
+    @retry_on_deadlock
+    def _set_resource_type_state(self, name, state):
+        with self.facade.writer() as session:
+            q = session.query(ResourceType)
+            q = q.filter(ResourceType.name == name)
+            update = q.update({'state': state})
+            if update == 0:
+                raise indexer.IndexerException(
+                    "Fail to set resource type state of %s to %s" %
+                    (name, state))
+
     @staticmethod
     def get_resource_type_schema():
         return base.RESOURCE_TYPE_SCHEMA_MANAGER
@@ -344,11 +417,28 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     # both adding the resource_type in our table and calling
     # map_and_create_tables() :-(
     @retry_on_deadlock
-    def _delete_resource_type(self, name):
+    def _mark_as_deleting_resource_type(self, name):
         try:
             with self.facade.writer() as session:
-                resource_type = self._get_resource_type(session, name)
-                session.delete(resource_type)
+                rt = self._get_resource_type(session, name)
+                if rt.state != "active":
+                    raise indexer.UnexpectedResourceTypeState(
+                        name, "active", rt.state)
+                session.delete(rt)
+
+                # FIXME(sileht): Why do I need to flush here !!!
+                # I want remove/add in the same transaction !!!
+                session.flush()
+
+                # NOTE(sileht): delete and recreate to:
+                # * raise duplicate constraints
+                # * ensure we do not create a new resource type
+                #   with the same name while we destroy the tables next
+                rt = ResourceType(name=rt.name,
+                                  tablename=rt.tablename,
+                                  state="deleting",
+                                  attributes=rt.attributes)
+                session.add(rt)
         except exception.DBReferenceError as e:
             if (e.constraint in [
                     'fk_resource_resource_type_name',
@@ -356,20 +446,40 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                     'fk_rh_resource_type_name']):
                 raise indexer.ResourceTypeInUse(name)
             raise
-        return resource_type
+        return rt
+
+    @retry_on_deadlock
+    def _delete_resource_type(self, name):
+        # Really delete the resource type, no resource can be linked to it
+        # Because we cannot add a resource to a resource_type not in 'active'
+        # state
+        with self.facade.writer() as session:
+            resource_type = self._get_resource_type(session, name)
+            session.delete(resource_type)
 
     def delete_resource_type(self, name):
         if name == "generic":
             raise indexer.ResourceTypeInUse(name)
 
-        resource_type = self._delete_resource_type(name)
+        rt = self._mark_as_deleting_resource_type(name)
 
-        with self.facade.writer_connection() as connection:
-            self._RESOURCE_TYPE_MANAGER.unmap_and_delete_tables(resource_type,
-                                                                connection)
+        try:
+            with self.facade.writer_connection() as connection:
+                self._RESOURCE_TYPE_MANAGER.unmap_and_delete_tables(
+                    rt, connection)
+        except Exception:
+            # NOTE(sileht): We fail the DDL, we have no way to automatically
+            # recover, just set a particular state
+            self._set_resource_type_state(rt.name, "deletion_error")
+            raise
+
+        self._delete_resource_type(name)
 
     def _resource_type_to_classes(self, session, name):
         resource_type = self._get_resource_type(session, name)
+        if resource_type.state != "active":
+            raise indexer.UnexpectedResourceTypeState(
+                name, "active", resource_type.state)
         return self._RESOURCE_TYPE_MANAGER.get_classes(resource_type)
 
     def list_archive_policies(self):
@@ -380,15 +490,37 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         with self.facade.independent_reader() as session:
             return session.query(ArchivePolicy).get(name)
 
+    def update_archive_policy(self, name, ap_items):
+        with self.facade.independent_writer() as session:
+            ap = session.query(ArchivePolicy).get(name)
+            if not ap:
+                raise indexer.NoSuchArchivePolicy(name)
+            current = sorted(ap.definition,
+                             key=operator.attrgetter('granularity'))
+            new = sorted(ap_items, key=operator.attrgetter('granularity'))
+            if len(current) != len(new):
+                raise indexer.UnsupportedArchivePolicyChange(
+                    name, 'Cannot add or drop granularities')
+            for c, n in zip(current, new):
+                if c.granularity != n.granularity:
+                    raise indexer.UnsupportedArchivePolicyChange(
+                        name, '%s granularity interval was changed'
+                        % c.granularity)
+            # NOTE(gordc): ORM doesn't update JSON column unless new
+            ap.definition = ap_items
+            return ap
+
     def delete_archive_policy(self, name):
+        constraints = [
+            "fk_metric_ap_name_ap_name",
+            "fk_apr_ap_name_ap_name"]
         with self.facade.writer() as session:
             try:
                 if session.query(ArchivePolicy).filter(
                         ArchivePolicy.name == name).delete() == 0:
                     raise indexer.NoSuchArchivePolicy(name)
             except exception.DBReferenceError as e:
-                if (e.constraint ==
-                   'fk_metric_archive_policy_name_archive_policy_name'):
+                if e.constraint in constraints:
                     raise indexer.ArchivePolicyInUse(name)
                 raise
 
@@ -438,31 +570,34 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     @retry_on_deadlock
     def create_metric(self, id, created_by_user_id, created_by_project_id,
                       archive_policy_name,
-                      name=None, resource_id=None):
+                      name=None, unit=None, resource_id=None):
         m = Metric(id=id,
                    created_by_user_id=created_by_user_id,
                    created_by_project_id=created_by_project_id,
                    archive_policy_name=archive_policy_name,
                    name=name,
+                   unit=unit,
                    resource_id=resource_id)
         try:
             with self.facade.writer() as session:
                 session.add(m)
         except exception.DBReferenceError as e:
             if (e.constraint ==
-               'fk_metric_archive_policy_name_archive_policy_name'):
+               'fk_metric_ap_name_ap_name'):
                 raise indexer.NoSuchArchivePolicy(archive_policy_name)
             raise
         return m
 
     @retry_on_deadlock
     def list_metrics(self, names=None, ids=None, details=False,
-                     status='active', **kwargs):
+                     status='active', limit=None, marker=None, sorts=None,
+                     **kwargs):
+        sorts = sorts or []
         if ids is not None and not ids:
             return []
         with self.facade.independent_reader() as session:
             q = session.query(Metric).filter(
-                Metric.status == status).order_by(Metric.id)
+                Metric.status == status)
             if names is not None:
                 q = q.filter(Metric.name.in_(names))
             if ids is not None:
@@ -472,6 +607,28 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             if details:
                 q = q.options(sqlalchemy.orm.joinedload('resource'))
 
+            sort_keys, sort_dirs = self._build_sort_keys(sorts)
+
+            if marker:
+                metric_marker = self.list_metrics(ids=[marker])
+                if metric_marker:
+                    metric_marker = metric_marker[0]
+                else:
+                    raise indexer.InvalidPagination(
+                        "Invalid marker: `%s'" % marker)
+            else:
+                metric_marker = None
+
+            try:
+                q = oslo_db_utils.paginate_query(q, Metric, limit=limit,
+                                                 sort_keys=sort_keys,
+                                                 marker=metric_marker,
+                                                 sort_dirs=sort_dirs)
+            except ValueError as e:
+                raise indexer.InvalidPagination(e)
+            except exception.InvalidSortKey as e:
+                raise indexer.InvalidPagination(e)
+
             return list(q.all())
 
     @retry_on_deadlock
@@ -605,12 +762,14 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 if update == 0:
                     raise indexer.NoSuchMetric(value)
             else:
+                unit = value.get('unit')
                 ap_name = value['archive_policy_name']
                 m = Metric(id=uuid.uuid4(),
                            created_by_user_id=r.created_by_user_id,
                            created_by_project_id=r.created_by_project_id,
                            archive_policy_name=ap_name,
                            name=name,
+                           unit=unit,
                            resource_id=r.id)
                 session.add(m)
                 try:
@@ -619,7 +778,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                     raise indexer.NamedMetricAlreadyExists(name)
                 except exception.DBReferenceError as e:
                     if (e.constraint ==
-                       'fk_metric_archive_policy_name_archive_policy_name'):
+                       'fk_metric_ap_name_ap_name'):
                         raise indexer.NoSuchArchivePolicy(ap_name)
                     raise
 
@@ -724,18 +883,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
                 q = q.filter(f)
 
-            # transform the api-wg representation to the oslo.db one
-            sort_keys = []
-            sort_dirs = []
-            for sort in sorts:
-                sort_key, __, sort_dir = sort.partition(":")
-                sort_keys.append(sort_key.strip())
-                sort_dirs.append(sort_dir or 'asc')
-
-            # paginate_query require at list one uniq column
-            if 'id' not in sort_keys:
-                sort_keys.append('id')
-                sort_dirs.append('asc')
+            sort_keys, sort_dirs = self._build_sort_keys(sorts)
 
             if marker:
                 resource_marker = self.get_resource(resource_type, marker)
@@ -753,9 +901,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             except ValueError as e:
                 raise indexer.InvalidPagination(e)
             except exception.InvalidSortKey as e:
-                # FIXME(jd) Wait for https://review.openstack.org/274868 to be
-                # released so we can return which key
-                raise indexer.InvalidPagination("Invalid sort keys")
+                raise indexer.InvalidPagination(e)
 
             # Always include metrics
             q = q.options(sqlalchemy.orm.joinedload("metrics"))
@@ -770,14 +916,19 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                         # No need for a second query
                         all_resources.extend(resources)
                     else:
+                        try:
+                            target_cls = self._resource_type_to_classes(
+                                session, type)['history' if is_history else
+                                               'resource']
+                        except (indexer.UnexpectedResourceTypeState,
+                                indexer.NoSuchResourceType):
+                            # NOTE(sileht): This resource_type have been
+                            # removed in the meantime.
+                            continue
                         if is_history:
-                            target_cls = self._resource_type_to_classes(
-                                session, type)['history']
-                            f = target_cls.revision.in_(
-                                [r.revision for r in resources])
+                            f = target_cls.revision.in_([r.revision
+                                                         for r in resources])
                         else:
-                            target_cls = self._resource_type_to_classes(
-                                session, type)["resource"]
                             f = target_cls.id.in_([r.id for r in resources])
 
                         q = session.query(target_cls).filter(f)
@@ -816,6 +967,23 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                     {"status": "delete"}) == 0:
                 raise indexer.NoSuchMetric(id)
 
+    @staticmethod
+    def _build_sort_keys(sorts):
+        # transform the api-wg representation to the oslo.db one
+        sort_keys = []
+        sort_dirs = []
+        for sort in sorts:
+            sort_key, __, sort_dir = sort.partition(":")
+            sort_keys.append(sort_key.strip())
+            sort_dirs.append(sort_dir or 'asc')
+
+        # paginate_query require at list one uniq column
+        if 'id' not in sort_keys:
+            sort_keys.append('id')
+            sort_dirs.append('asc')
+
+        return sort_keys, sort_dirs
+
 
 class QueryTransformer(object):
     unary_operators = {
