@@ -13,9 +13,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import logging
 import operator
 from oslo_config import cfg
+from oslo_log import log
 from oslo_utils import timeutils
 from stevedore import driver
 
@@ -28,16 +28,20 @@ OPTS = [
                default='file',
                help='Storage driver to use'),
     cfg.IntOpt('metric_processing_delay',
-               default=5,
+               default=10,
                help="How many seconds to wait between "
-               "new metric measure processing"),
+               "scheduling new metrics to process"),
     cfg.IntOpt('metric_reporting_delay',
                default=60,
                help="How many seconds to wait between "
                "metric ingestion reporting"),
+    cfg.IntOpt('metric_cleanup_delay',
+               default=60,
+               help="How many seconds to wait between "
+               "cleaning of expired data"),
 ]
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 class Measure(object):
@@ -168,11 +172,10 @@ class StorageDriver(object):
     def upgrade(index):
         pass
 
-    def process_background_tasks(self, index, block_size=128, sync=False):
+    def process_background_tasks(self, index, metrics, sync=False):
         """Process background tasks for this storage.
 
-        This calls :func:`process_measures` to process new measures and
-        :func:`expunge_metrics` to expunge deleted metrics.
+        This calls :func:`process_new_measures` to process new measures
 
         :param index: An indexer to be used for querying metrics
         :param block_size: number of metrics to process
@@ -180,24 +183,24 @@ class StorageDriver(object):
                      on error
         :type sync: bool
         """
-        LOG.debug("Processing new and to delete measures")
+        LOG.debug("Processing new measures")
         try:
-            self.process_measures(index, block_size, sync)
+            self.process_new_measures(index, metrics, sync)
         except Exception:
             if sync:
                 raise
             LOG.error("Unexpected error during measures processing",
                       exc_info=True)
-        LOG.debug("Expunging deleted metrics")
-        try:
-            self.expunge_metrics(index, sync)
-        except Exception:
-            if sync:
-                raise
-            LOG.error("Unexpected error during deleting metrics",
-                      exc_info=True)
 
     def expunge_metrics(self, index, sync=False):
+        """Remove deleted metrics
+
+        :param index: An indexer to be used for querying metrics
+        :param sync: If True, then delete everything synchronously and raise
+                     on error
+        :type sync: bool
+        """
+
         metrics_to_expunge = index.list_metrics(status='delete')
         for m in metrics_to_expunge:
             try:
@@ -225,7 +228,7 @@ class StorageDriver(object):
         raise exceptions.NotImplementedError
 
     @staticmethod
-    def process_measures(indexer=None, block_size=None, sync=False):
+    def process_new_measures(indexer, metrics, sync=False):
         """Process added measures in background.
 
         Some drivers might need to have a background task running that process
