@@ -16,16 +16,13 @@
 # under the License.
 import collections
 import datetime
-import logging
-import multiprocessing
 import operator
-import threading
-import time
 import uuid
 
 from concurrent import futures
 import iso8601
 from oslo_config import cfg
+from oslo_log import log
 from oslo_serialization import msgpackutils
 from oslo_utils import timeutils
 import six
@@ -36,15 +33,17 @@ from gnocchi import storage
 
 OPTS = [
     cfg.IntOpt('aggregation_workers_number',
+               default=1, min=1,
                help='Number of workers to run during adding new measures for '
-                    'pre-aggregation needs.'),
+                    'pre-aggregation needs. Due to the Python GIL, '
+                    '1 is usually faster, unless you have high latency I/O'),
     cfg.StrOpt('coordination_url',
                secret=True,
                help='Coordination driver URL'),
 
 ]
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 class CarbonaraBasedStorage(storage.StorageDriver):
@@ -56,33 +55,12 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             self.coord = coordination.get_coordinator(
                 conf.coordination_url,
                 str(uuid.uuid4()).encode('ascii'))
-            self.coord.start()
+            self.coord.start(start_heart=True)
         except Exception as e:
             raise storage.StorageError("Unable to start coordinator: %s" % e)
-        if conf.aggregation_workers_number is None:
-            try:
-                self.aggregation_workers_number = multiprocessing.cpu_count()
-            except NotImplementedError:
-                self.aggregation_workers_number = 2
-        else:
-            self.aggregation_workers_number = conf.aggregation_workers_number
-        self.partition = 0
-        self._stop_heartbeat = threading.Event()
-        self.heartbeater = threading.Thread(target=self._heartbeat,
-                                            name='heartbeat')
-        self.heartbeater.setDaemon(True)
-        self.heartbeater.start()
-
-    def _heartbeat(self):
-        while not self._stop_heartbeat.is_set():
-            # FIXME(jd) Why 10? Why not. We should have a way to find out
-            # what's the best value here, but it depends on the timeout used by
-            # the driver; tooz should help us here!
-            time.sleep(10)
-            self.coord.heartbeat()
+        self.aggregation_workers_number = conf.aggregation_workers_number
 
     def stop(self):
-        self._stop_heartbeat.set()
         self.coord.stop()
 
     def _lock(self, metric_id):
@@ -102,7 +80,8 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         raise NotImplementedError
 
     @staticmethod
-    def _store_metric_measures(metric, aggregation, granularity, data):
+    def _store_metric_measures(metric, timestamp_key,
+                               aggregation, granularity, data):
         raise NotImplementedError
 
     @staticmethod
@@ -224,11 +203,11 @@ class CarbonaraBasedStorage(storage.StorageDriver):
                 oldest_point_to_keep)
 
     def add_measures(self, metric, measures):
-        self._store_measures(metric, msgpackutils.dumps(
+        self._store_new_measures(metric, msgpackutils.dumps(
             list(map(tuple, measures))))
 
     @staticmethod
-    def _store_measures(metric, data):
+    def _store_new_measures(metric, data):
         raise NotImplementedError
 
     @staticmethod
@@ -236,7 +215,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
         raise NotImplementedError
 
     @staticmethod
-    def _list_metric_with_measures_to_process(full=False):
+    def list_metric_with_measures_to_process(size, part, full=False):
         raise NotImplementedError
 
     @staticmethod
@@ -317,9 +296,7 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             self._check_for_metric_upgrade,
             ((metric,) for metric in index.list_metrics()))
 
-    def process_measures(self, indexer, block_size, sync=False):
-        metrics_to_process = self._list_metric_with_measures_to_process(
-            block_size, full=sync)
+    def process_new_measures(self, indexer, metrics_to_process, sync=False):
         metrics = indexer.list_metrics(ids=metrics_to_process)
         # This build the list of deleted metrics, i.e. the metrics we have
         # measures to process for but that are not in the indexer anymore.
@@ -455,9 +432,9 @@ class CarbonaraBasedStorage(storage.StorageDriver):
             )
             granularities_in_common = [
                 g
-                for g, occurence in six.iteritems(
+                for g, occurrence in six.iteritems(
                     collections.Counter(granularities))
-                if occurence == len(metrics)
+                if occurrence == len(metrics)
             ]
 
             if not granularities_in_common:
